@@ -3,14 +3,16 @@ package Dancer::Route::Cache;
 use strict;
 use warnings;
 use Carp;
+
+use base 'Dancer::Object';
+
 use vars '$VERSION';
 
-use Dancer::Object;
 use Dancer::Config 'setting';
 use Dancer::Error;
+use Dancer::Exception qw(:all);
 
 $VERSION = '0.01';
-use base 'Dancer::Object';
 Dancer::Route::Cache->attributes('size_limit', 'path_limit');
 
 # static
@@ -70,35 +72,39 @@ sub parse_size {
 }
 
 sub route_from_path {
-    my ($self, $method, $path) = @_;
+    my ($self, $method, $path, $app_name) = @_;
 
     $method && $path
-      or croak "Missing method or path";
+      or raise core_route => "Missing method or path";
 
-    return $self->{'cache'}{$method}{$path} || undef;
+    $app_name = 'main' unless defined $app_name;
+
+    return $self->{'cache'}{$app_name}{$method}{$path} || undef;
 }
 
 sub store_path {
-    my ($self, $method, $path, $route) = @_;
+    my ($self, $method, $path, $route, $app_name) = @_;
 
     $method && $path && $route
-      or croak "Missing method, path or route";
+      or raise core_route => "Missing method, path or route";
 
-    $self->{'cache'}{$method}{$path} = $route;
+    $app_name = 'main' unless defined $app_name;
 
-    push @{$self->{'cache_array'}}, [$method, $path];
+    $self->{'cache'}{$app_name}{$method}{$path} = $route;
+
+    push @{$self->{'cache_array'}}, [$method, $path, $app_name];
 
     if (my $limit = $self->size_limit) {
         while ($self->route_cache_size() > $limit) {
-            my ($method, $path) = @{shift @{$self->{'cache_array'}}};
-            delete $self->{'cache'}{$method}{$path};
+            my ($method, $path, $app_name) = @{shift @{$self->{'cache_array'}}};
+            delete $self->{'cache'}{$app_name}{$method}{$path};
         }
     }
 
     if (my $limit = $self->path_limit) {
         while ($self->route_cache_paths() > $limit) {
-            my ($method, $path) = @{shift @{$self->{'cache_array'}}};
-            delete $self->{'cache'}{$method}{$path};
+            my ($method, $path, $app_name) = @{shift @{$self->{'cache_array'}}};
+            delete $self->{'cache'}{$app_name}{$method}{$path};
         }
     }
 }
@@ -110,15 +116,20 @@ sub route_cache_size {
 
     use bytes;
 
-    foreach my $method (keys %cache) {
-        $size += length $method;
+    foreach my $app_name (keys %cache) {
+        $size += length $app_name;
 
-        foreach my $path (keys %{$cache{$method}}) {
-            $size += length $path;
-            $size += length $cache{$method}{$path};
+        foreach my $method (keys %{$cache{$app_name}}) {
+            $size += length $method;
+
+            foreach my $path (keys %{$cache{$app_name}{$method}}) {
+                $size += length $path;
+                $size += length $cache{$app_name}{$method}{$path};
+            }
         }
     }
 
+
     no bytes;
 
     return $size;
@@ -128,7 +139,7 @@ sub route_cache_paths {
     my $self = shift;
     my %cache = $self->{'cache'} ? %{$self->{'cache'}} : ();
 
-    return scalar map { keys %{$cache{$_}} } keys %cache;
+    return scalar map { keys %{$_} } map { values %{$cache{$_}} } keys %cache;
 }
 
 1;
@@ -159,7 +170,7 @@ the requested path to a route.
 A major drawback is that L<Dancer> has to go over the matching on every request,
 which (especially on CGI-based applications) can be very time consuming.
 
-The caching mechanism allows to cache some requests to specific routes (but
+The caching mechanism allows one to cache some requests to specific routes (but
 B<NOT> specific results) and run those routes on a specific path. This allows us
 to speed up L<Dancer> quite a lot.
 
@@ -208,12 +219,15 @@ size limit.
 =head2 route_cache_paths
 
 Returns all the paths in the cache. This is used to enforce the path limit.
+Please be carefull if you use L<Plack::Builder/mount> and some applications -
+routes are linked with applications and same path may be in some applications
+but with different handlers!
 
 =head1 ATTRIBUTES
 
 =head2 size_limit($limit)
 
-Allows to set a size limit of the cache.
+Allows one to set a size limit of the cache.
 
 Returns the limit (post-set).
 
@@ -242,4 +256,3 @@ under the terms of either: the GNU General Public License as published
 by the Free Software Foundation; or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
-
